;======================== header ===============================================
;+
; NAME:
;   mk42zip
; PURPOSE:
;   make zip files for Solar Weather Browser for MLSO Mark-IV images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;   date: yyyymmdd
; MODIFICATION HISTORY:
; Created C.Marque, ROB, Feb 21, 2007. Inspired by original LASCO2zip
;         2007/05/14: clean up code, make custom output directory for SECCHI 
;                     SWB archives at ROB                          
; Andrew L. Stanger, ROB, 29 May 2007: Adapted from "cor2zip.pro".
; 14 Jun 2007 [ALS]: Display Mk4 image in IDL window.
; 20 Jun 2007 [ALS]: Adapt to HAO computing environment.
; 17 Jun 2007 [ALS]: Set occulting disc & outer FOV pixels to zero.
;-
;======================== end header ===========================================

PRO mk42zip, logfile=logfile, date=date

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'mk42zip'

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter.
; ------------------------------------------------------------------------------

year  = strmid (date, 0, 4)
month = strmid (date, 4, 2)
day   = strmid (date, 6, 2)

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

set_swbs
;#datadir = !swbs.data_dir + year + '/' + month + '/' + day + '/'

datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'
print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; Say hello.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; Reference information.
; ------------------------------------------------------------------------------

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

window, 0, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Get list of good images
; ------------------------------------------------------------------------------

sdir = datadir + '*mk4.rpb.vig.fts*'
print, 'sdir: ', sdir

;list = file_search (datadir + '*mk4.rpb.fts*')

spawn, 'ls ' + sdir, list

; ------------------------------------------------------------------------------
; Continue only if list is not empty.
; ------------------------------------------------------------------------------

IF (list [0] EQ ' ') THEN printf, log, '  No images available for this period' $
ELSE BEGIN ;{ ------- list not empty -------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]
    print, 'fitsfile: ', fitsfile

    ; --------------------------------------------------------------------------
    ; Report starting processing image
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'Processing image ', list[i]
    printf, log, ''

    ; --------------------------------------------------------------------------
    ; Read image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "mk42zip: Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    mk4_date = index.date_obs
    mk4_time = index.time_obs
    date     = index.date_obs + "T" + index.time_obs

    year     = strmid (mk4_date,  0, 4)
    month    = strmid (mk4_date,  5, 2)
    day      = strmid (mk4_date,  8, 2)

    hours    = strmid (mk4_time,  0, 2)
    mins     = strmid (mk4_time,  3, 2)
    sec      = strmid (mk4_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Solar radius in arcsec.
    ; --------------------------------------------------------------------------

    sol_rad = index.rsun

    ; --------------------------------------------------------------------------
    ; Image brightness min/max values.
    ; --------------------------------------------------------------------------

    dispmin = index.dispmin		; display minimum.
    dispmax = index.dispmax		; display maximum.

    ; --------------------------------------------------------------------------
    ; Re-size image.
    ; --------------------------------------------------------------------------

    icolor =   0
    mag = float (outdim) / float (index.naxis1)	; Magnification factor.
    oxcen = index.crpix1 * mag				; x-axis sun center.
    oycen = index.crpix2 * mag				; y-axis sun center.
    opixrs = index.crradius * mag			; pixels/Rsun

    print, 'naxis1:   ', index.naxis1
    print, 'mag:      ', mag
    print, 'crradius: ', index.crradius
    print, 'opixrs:   ', opixrs

    img = img > 0
    img = congrid (img, outdim, outdim)		; Re-size imimage.

    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness, using dispmin & dispmax values.
    ; --------------------------------------------------------------------------

;    img   = bytscl (img, dispmin, dispmax)		; Re-scale brightness.

    ; For now, use a high-contrast linear scaling of brightness.
 
    dmin = - 50
    dmax =  300

    img   = bytscl (img, dmin, dmax)		; Re-scale brightness.
    mask_im = where (img LE min (img))		; Clip dim values.
    img [mask_im] = 0  

    ; --------------------------------------------------------------------------
    ; Smooth image.
    ; --------------------------------------------------------------------------

    img = smooth (img, 3)

    ; --------------------------------------------------------------------------
    ; Set pixels inside occulting disk and outer FOV to zero.
    ; --------------------------------------------------------------------------

    img = corona_mask (img, icolor, outdim, outdim, oxcen, oycen, opixrs, $
                       rmin=1.15, rmax=2.8)

    ; --------------------------------------------------------------------------
    ; Display image.
    ; --------------------------------------------------------------------------

    tv, img

    ; --------------------------------------------------------------------------
    ; Create outdir, outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn = 'mlso_mk4_' + year + month + day + '_' + hours + mins + sec
    outdir = !swbs.out_dir + 'mlso/' + year + '/' + month + '/' + day + '/'
    outdir = '/archive/swbs/' + 'mlso/' + year + '/' + month + '/' + day + '/'
    data_type = 'C'
    data_name = 'MLSO Mk4'
    color_table_code = '001' 

    ; --------------------------------------------------------------------------
    ; SWB's standard Field Of View (FOV).
    ; --------------------------------------------------------------------------

    swb_fov = sol_rad * outdim / ref_solradius  

    ; --------------------------------------------------------------------------
    ; Calculate the ratio of COR's FOV to the standard SWB's FOV.
    ; --------------------------------------------------------------------------

    mk4_fov = index.naxis1 * index.cdelt1 
    fov_ratio = mk4_fov / swb_fov

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    write2zip, outfn, outdir, img, index, data_type, data_name, $
               color_table_code, date, fov_ratio=fov_ratio, /logfile
    ; cd, !swbs.source_dir + 'bin/' + myname
    printf, log, '  processed mk4 image ', list [i]

    ENDFOR ;} i
    ; --------------------------------------------------------------------------

ENDELSE	;} ------- list not empty -------------

; ------------------------------------------------------------------------------
; Say bye-bye.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, ' <<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

IF (keyword_set (logfile)) THEN free_lun, log   
END
